// compressfile.h: interface for the CCompressFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(COMPRESSFILE__INCLUDED_)
#define COMPRESSFILE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "DataFile.h"
#include "PackFile.h"

namespace FileSystem
{

	struct TPacksListElement;

//!  Compresed data file
/*!
	This virtual class its the implemetation
	for the packer of compresed files, those file
	are mapped to a file inside a pack file.

	\ingroup Base
	\sa CDataFile
	\sa CPhysicalFile
*/
class CCompressFile : public CDataFile  
{
public:

	//! CDataFile class it's the parent of CCompressFile class
	/*!
	    CDataFile class it's the parent of CCompressFile class
	*/
	typedef CDataFile Inherit;


	//!  Default constructor. 
	/*!
		Called before the object creation.
	*/
	inline CCompressFile():Inherit(){};

	//!  Default destructor. 
	/*!
		Called before the object destruction.
	*/
	virtual ~CCompressFile(){};

	//!  Initialize an CCompressFile Object.
	/*!
		This method initialize the CCompressFile object.
		\retval CResult value indicate if the operation was correct
	*/
	 CResult initialize(CDataManager* pManager,TPacksListElement* pack,
			TFileInfoElement* element);
										

	//!  Finalize an CCompressFile Object.
	/*!
		This method finalize the CCompressFile object.
		\retval CResult value indicate if the operation was correct
	*/
	virtual CResult finalize();
	
	//! To Know if we are in the end of the file
	/*!
		To Know if we are in the end of the file
		\retval if we are at the end of the file
	*/
	int eof();

	//! Seek an amout of byte in the file
	/*!
		Seek an amout of byte in the file
		\param amout = amout of bytes to be seek
		\pos = SEEK_CUR, SEEK_END, SEEK_SET
		\retval 0 if its correct
	*/
	int seek(long amount,int pos);

	//! Read an amout of byte in the file
	/*!
		Read an amout of byte in the file
		\param amout = amout of bytes to read
		\param buffer* = destination buffer
		\retval number of bytes readed
	*/
	int read(long amount,char* buffer);


	//!  Read a string into the buffer
	/*!
		You should call this function to read a
		string into a buffer

		\param maxLen = maximum string size.
		\param buffer = a pointer of a buffer to
		 be fille with the data		 
		 \retval the string itself or NULL if fail
	*/
	char* gets(char* buffer,int maxLen);
private:

	//! Read and uncompress a block into the memory
	/*!
		Read an block into memory. If the block
		its allready in memory skip the read
		\param num = number of the block				
	*/
	CResult readBlock(int num);

	//! The block size
	unsigned long int blockSize;

	//! Uncompress readed buffer
	unsigned char* buffer;
	//! Compress readed buffer
	unsigned char* readBuffer;

	//! The last block readed
	signed long int lastBlockRead;

	//! Total number of blocks in the file
	unsigned long numBlocks;
	//! Array of block offsets
	unsigned long* blockOffsets;

	//! Current file position
	unsigned long int currentPos;

	//! Total size of the file
	unsigned long int totalSize;

	//! Offset from the begining of the blocks
	unsigned long int offset;

	//! Internal file handle
	FILE* file;
};

}
#endif // !defined(COMPRESSFILE__INCLUDED_)

