// DataFile.h: interface for the CDataFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(DATAFILE__INCLUDED_)
#define DATAFILE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BaseObject.h"

namespace FileSystem
{
	class CDataManager;
//!  Class for handle data files
/*!
	This class it's used for handle data files.
	\ingroup Base
	\sa CCompressFile
	\sa CPhysicalFile
*/
class CDataFile : public CBaseObject  
{
public:

	//! CBaseObject class it's the parent of CDataFile class
	/*!
	    CBaseObject class it's the parent of CDataFile class
	*/
	typedef CBaseObject Inherit;


	//!  Default constructor. 
	/*!
		Called before the object creation.
	*/
	inline CDataFile(){Inherit();};

	//!  Default destructor. 
	/*!
		Called before the object destruction.
	*/
	virtual ~CDataFile(){};

	//!  Initialize an CDataFile Object.
	/*!
		This method initialize the CDataFile object.
		\retval CResult value indicate if the operation was correct
	*/
	virtual CResult initialize(CDataManager* pManager);

	//!  Finalize an CDataFile Object.
	/*!
		This method finalize the CDataFile object.
		\retval CResult value indicate if the operation was correct
	*/
	virtual CResult finalize(){return Inherit::finalize();};
	
	//!  Seek file pointer in a datafile.
	/*!
		With this method you could seek the file
		position insinde de data file.
		\param amount = amount of bytes to seak		
		\param pos = starting pos SEEK_CUR=from current position,SEEK_END=from end of file,SEEK_SET=beginning of file.
	*/
	virtual int seek(long amount,int pos)=0;

	//!  Read and amout of bytes in a data file
	/*!
		You should call this function to read an
		amount of bytes in a data file.

		\param amount = amount of bytes to read.
		\param buffer = a pointer of a buffer to
		 be fille with the data		 
		 \retval amount of byte readed
	*/
	virtual int read(long amount,char* buffer)=0;

	//! Determinate if its the end of the file
	/*!
		You should call this function to know if
		you have reach the end of the file.
		 \retval non zero if you reach the end
	*/
	virtual int eof()=0;

	//!  Read a string into the buffer
	/*!
		You should call this function to read a
		string into a buffer

		\param maxLen = maximum string size.
		\param buffer = a pointer of a buffer to
		 be fille with the data		 
		 \retval the string itself or NULL if fail
	*/
	virtual char* gets(char* buffer,int maxLen)=0;
protected:

	//! The data manager store as a member
	CDataManager* m_pManager;
};
}
#endif // !defined(DATAFILE__INCLUDED_)
